#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <gui/window.h>
#include <gui/view.h>

#include <khtml_part.h>
#include <khtmlview.h>
#include <misc/loader.h>
#include <klibloader.h>
#include <ecma/kjs_proxy.h>
#include <kiconloader.h>

#include <string>
#include <list>
#include <vector>

#include <atheos/amutex.h>
#include <util/message.h>
#include <util/messenger.h>


using namespace os;

enum {
	 ID_STOP,
	 ID_URLVIEW_CHANGE,
	 ID_URLVIEW_ENTER,
	 ID_TAB_CHANGED,
	 ID_NEW_TAB,
	 ID_DELETE_TAB,
	 ID_OPEN_TAB,
    ID_URL_CHANGED,
    ID_OPEN_URL,
    ID_GOHOME,
    ID_NEXT_URL,
    ID_PREV_URL,
    ID_RELOAD,
    ID_FIND,
    ID_REPEAT_FIND,
    ID_DO_SEARCH,
    ID_ACTIVATE_URLEDIT,
    ID_SEARCH_CLOSED,
    ID_START_DOWNLOAD,
    ID_CANCEL_DOWNLOAD,
    ID_NEW_WINDOW,
    ID_OPEN_LINK,
    ID_SAVE_LINK,
    ID_COPY_LINK_LOCATION,
    ID_COPY,    
	 ID_PASTE,
    ID_VIEW_DOC_SRC,
    ID_VIEW_FRAME_SRC
};

enum { BI_BACK, BI_FORWARD, BI_GOHOME, BI_RELOAD, BI_FIND, BI_COPY };

// ### from ecma/kjs_proxy.cpp and khtml_factory.cpp
extern "C"
{
    KJSProxy *kjs_html_init( KHTMLPart * );
    void *init_libkhtml();
}

// ### from khtml_part.cpp
extern int kjs_lib_count;


struct HistoryEntry
{
    os::Message m_cState;
    std::string m_cURL;
};

class TopView : public QWidget
{
public:
    TopView() : QWidget( NULL, "top_view" ) {}

    virtual void KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers )
    {
	if ( nQualifiers & QUAL_CTRL ) {
	    switch( pzRawString[0] )
	    {
		case VK_TAB:
		    GetWindow()->PostMessage( ID_ACTIVATE_URLEDIT, GetWindow() );
		    break;
		case 'r':
		    GetWindow()->PostMessage( ID_RELOAD, GetWindow() );
		    break;		    
		case 'f':
		    GetWindow()->PostMessage( ID_FIND, GetWindow() );
		    break;
		case VK_INSERT:
		case 'c':
		    GetWindow()->PostMessage( ID_COPY, GetWindow() );
		    break;
		case 'n':
		    GetWindow()->PostMessage( ID_NEW_WINDOW, GetWindow() );
		    break;
		default:
		    QWidget::KeyDown( pzString, pzRawString, nQualifiers );
		    break;
	    }
	} else {
	    switch( pzRawString[0] )
	    {
		case VK_BACKSPACE:
		    GetWindow()->PostMessage( ID_PREV_URL, GetWindow() );
		    break;		    
		case VK_FUNCTION_KEY:
		{
		    os::Message* pcMsg = GetLooper()->GetCurrentMessage();
		    assert( pcMsg != NULL );
		    int32 nKeyCode;

		    if ( pcMsg->FindInt32( "_raw_key", &nKeyCode ) != 0 ) {
			return;
		    }
	
		    switch( nKeyCode )
		    {
			case 4:	// F3
			    GetWindow()->PostMessage( ID_REPEAT_FIND, GetWindow() );
			    break;
			default:
			    QWidget::KeyDown( pzString, pzRawString, nQualifiers );
			    break;
		    }
		    break;
		}
		default:
		    QWidget::KeyDown( pzString, pzRawString, nQualifiers );
		    break;
		
	    }
	}
    }
    
};

class HTMLView : public TopView
{
public:
	HTMLView();
	HTMLView(const std::string& cURL, const KParts::URLArgs& cArgs);
	KHTMLPart* getParts();
	void setPart(KHTMLPart *temp);
	void setCHP(int temp);
	int getCHP();
	HistoryEntry* HistoryCurrent();
	std::vector<HistoryEntry*> getHE();
	void openURL(const std::string& cURL, const KParts::URLArgs& cArgs);
	void AddHistoryEntry();
	void GoHistory( int nStep);
	void UpdateHistory(HistoryEntry *temp);
private:
	KHTMLPart* kpart;
	int		m_nCurHistoryPos;	
	std::vector<HistoryEntry*> m_cHistory;
};

