#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/stat.h>

#include <atheos/time.h>

#include <util/application.h>
#include <util/message.h>
#include <util/messenger.h>
#include <util/clipboard.h>
#include <gui/window.h>
#include <gui/view.h>
#include <gui/frameview.h>
#include <gui/button.h>
#include <gui/textview.h>
#include <gui/tabview.h>
#include <gui/listview.h>


#include <misc/loader.h>


#include <string>
#include <list>
#include <vector>

#include <gui/menu.h>
#include <gui/requesters.h>
#include <storage/path.h>
#include <storage/file.h>
#include <storage/directory.h>
#include <storage/tempfile.h>

#include <atheos/amutex.h>

#include "toolbar.h"
#include "statusbar.h"
#include "searchdialog.h"
#include "downloaddialog.h"
#include "favoritemenu.h"
#include "dropmenu.h"
#include "HTMLView.h"

typedef vector <std::string> t_List;

using namespace os;

static Point get_window_pos()
{
    static Point cLastPos( 100, 100 );
    cLastPos.x += 30;
    cLastPos.y += 30;
    if ( cLastPos.x > 500 )
    {
        cLastPos.x = 100;
    }
    if ( cLastPos.y > 500 )
    {
        cLastPos.y = 100;
    }
    return( cLastPos );
}



#define ID_CANCEL_FAV_WINDOW 0x43
#define ID_OK_FAV_WINDOW  0x44

const char* g_pzDefaultURL = "file:///Applications/ABrowse/docs/startup.html";



struct DownloadNode
{
    DownloadNode()
    {
        m_psDownloadStream = NULL;
        m_pcRequester      = NULL;
        m_pcProgressDlg    = NULL;
        m_nContentSize     = -1;
        m_pcTmpFile        = NULL;
        m_pcFile           = NULL;
        m_nBytesReceived   = 0;
        m_bDone            = false;
        m_bCanceled        = false;
    }

    KParts::DownloadStream* m_psDownloadStream;
    DownloadRequester*	    m_pcRequester;
    DownloadProgress*	    m_pcProgressDlg;
    std::string		    m_cURL;
    std::string		    m_cPreferredName;
    off_t		    m_nContentSize;
    bigtime_t		    m_nStartTime;
    std::string	            m_cPath;
    os::TempFile*           m_pcTmpFile;
    os::File*	            m_pcFile;
    int		            m_nBytesReceived;
    bool		    m_bDone;
    bool	            m_bCanceled;
};


class Tabs : public TabView
{
public:
    Tabs(const Rect& r);
};

Tabs :: Tabs(const Rect& r) : TabView(r, "Testing Tabs",CF_FOLLOW_ALL,WID_WILL_DRAW|WID_CLEAR_BACKGROUND)
{}

class FavoriteWindow : public Window
{
public:
    FavoriteWindow(string zUrl, string zTitle, Window* pcParent);
    void CenterWindow(Window* pcWin);
private:
    StringView* pcStringView;
    TextView* pcTextView;
    Button* pcOk;
    Button* pcCancel;
    virtual void HandleMessage(Message* pcMessage);
    std::string sUrl;
    std::string sFileName;
    void SaveFile();
    Window* pcParentWindow;

};

class ManageFavorites : public Window
{
public:
	ManageFavorites(t_List list, Window* pcParent);
	void CenterWindow(Window* pcWin);
private:
	
};

class BrowserWindow : public Window, public KParts::BrowserCallback
{
public:
    BrowserWindow( const Rect& cFrame, bool bLoadPos );
    ~BrowserWindow();

    void OpenURL( const std::string& cURL, const KParts::URLArgs& cArgs );
    void CreateNewTab();
    void AddFavorites();
    t_List GetFavorites();
    void UpdateFavorites();
    void CreateNewTab(const std::string& cURL, const KParts::URLArgs& cArgs);
    void SwitchTab();
    void DeleteTab();
    virtual void DispatchMessage( Message* pcMessage, Handler* pcHandler )
    {
        GlobalMutex::PushLooper( this );
        Window::DispatchMessage( pcMessage, pcHandler );
        GlobalMutex::PopLooper();
    }
    virtual void HandleMessage( Message* pcMsg );

    // From Window:
    bool	OkToQuit();
#if 0

    virtual void TimerTick( int /*nID*/ )
    {
        //	Unlock();
        GlobalMutex::Lock();
        GlobalMutex::PushLooper( this );
        //	Lock();
        khtml::Loader::Run();
        GlobalMutex::PopLooper();
        GlobalMutex::Unlock();
    }
#endif
    // From BrowserCallback:
    virtual void EnableAction( const char* /*name*/, bool /*enabled*/ )
    {}
    virtual void OpenURLRequest( const std::string &url, const KParts::URLArgs& args );
    virtual void OpenURLNotify();
    virtual void Completed();
    virtual void SetLocationBarURL( const std::string &url );
    virtual void CreateNewWindow( const std::string& url, const KParts::URLArgs& args );
    virtual KParts::ReadOnlyPart* CreateNewWindow( const std::string& url, const KParts::URLArgs& args,
            const KParts::WindowArgs& windowArgs );
    virtual void PopupMenu( KParts::BrowserExtension* pcSource, const std::string& cURL );
    virtual void LoadingProgress( int percent );
    virtual void SpeedProgress( int bytesPerSecond );
    virtual void InfoMessage( const std::string& cString );
    virtual void SetWindowCaption( KParts::ReadOnlyPart* pcPart, const std::string &caption );
    virtual void SetStatusBarText( const std::string &text );
    virtual void SelectionInfo( const std::string& text );

    virtual bool BeginDownload( KParts::DownloadStream* psStream );
    virtual bool HandleDownloadData( KParts::DownloadStream* psStream, const void* pData, int nLen );
    virtual void DownloadFinished( KParts::DownloadStream* psStream, int nError );

    // New members:

    DropMenu* GetURLView();
    KHTMLPart*	GetPart();
private:
    static int s_nWndCount;

    void	  UpdateHistory();
    void	  AddHistoryEntry();
    void	  GoHistory( int nStep );
    void _Init();
    std::string	m_cSelectedText;
    std::string m_cLastSearchStr;
    bool	m_bSearchCaseSensitive;
    ToolBar*   m_pcToolBar;
    StatusBar* m_pcStatusBar;
    DropMenu*  m_pcURLView;
    os::Menu*  m_pcPopupMenu;
    FavoriteMenu* pcFavoritesMenu;
    Tabs* m_pcTabs;
    FindDialog* m_pcSearchDialog;
    bool        m_bSaveSize;
    Message* pcFavoriteMessage;
    FavoriteWindow* pcFavoriteWindow;

};

class	MyApp : public Application
{
public:
    MyApp( const char* pzName );
    ~MyApp();

    // From Handler:
    virtual void	HandleMessage( Message* pcMsg );
    virtual void TimerTick( int /*nID*/ )
    {
        //	Unlock();
        //	GlobalMutex::Lock();
        //	Lock();
        GlobalMutex::PushLooper( this );
        khtml::Loader::Run();
        GlobalMutex::PopLooper();
        //	GlobalMutex::Unlock();
    }

private:

};

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

MyApp::MyApp( const char* pzName ) : Application( pzName )
{
    //    signal( SIGCHLD, SIG_IGN );
	string cString = (string) getenv("HOME");
	mkdir ((cString +  (string)"/Settings").c_str(),0700);
    mkdir ((cString +  (string)"/Settings/ABrowse").c_str(),0700);
    mkdir ((cString +  (string)"/Settings/ABrowse/Favorites").c_str(),0700);
    
    GlobalMutex::Lock();
    SetMutex( GlobalMutex::GetMutex() );

    kjs_lib_count++; // ### hack, prevent khtmlpart from deleting the js klibrary object

    KLibLoader::self()->library( "kjs_html" )->registerSymbol( "kjs_html_init", (void *)&kjs_html_init );
    KLibLoader::self()->library( "libkhtml" )->registerSymbol( "init_libkhtml", (void *)&init_libkhtml );

    BrowserWindow* pcWindow = new BrowserWindow( Rect( 150, 200, 1000, 800 ), true );

    KParts::URLArgs cArgs;

    /*try {
    std::string cDefaultPath;
    Directory cDir( open_image_file( IMGFILE_BIN_DIR ) );
    cDir.GetPath( &cDefaultPath );
    cDefaultPath = std::string("file:") + cDefaultPath + "/docs/startup.html";
    pcWindow->OpenURL( cDefaultPath, cArgs );
    } catch(...) {
    }*/

    pcWindow->Show();
    pcWindow->MakeFocus();
    AddTimer( this, 1, 100000, false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

MyApp::~MyApp()

{}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void MyApp::HandleMessage( Message* pcMsg )
{
    switch( pcMsg->GetCode() )
    {

    default:
        Application::HandleMessage( pcMsg );
        return;
    }
}

ManageFavorites::ManageFavorites(t_List list, Window* pcParent) : Window(Rect(0,0,600,400), "_fav_manage", "Manage Favorites...",0)
{
ListView* lview = new ListView(GetBounds(),"list");
AddChild(lview);
cout<<"launching favorite manager"<<endl;
cout<<getenv("HOME")<<endl;
for (uint32 n = 0; n < list.size(); n++)
    {
		ListViewStringRow* temp = new ListViewStringRow();
		cout<<"test"<<endl;
		temp->AppendString(list[n]);
		cout<<"test2"<<endl;
		lview->InsertRow(temp,true);
		cout<<list[n].c_str()<<endl;
    }
}

void ManageFavorites::CenterWindow(Window* pcWin)
{
Rect cBounds = GetBounds();
Rect cR = pcWin->GetFrame();
MoveTo(cR.left + (cR.Width() - cBounds.Width() ) /2, cR.top + (cR.Height() - cBounds.Height())/2);
}

FavoriteWindow::FavoriteWindow(string zUrl, string zTitle, Window* pcParent) : Window(Rect(0, 0, 300, 70), "_fav_window", "Add to Favorites...", WND_NOT_MOVEABLE | WND_NOT_RESIZABLE)
{
    sUrl = zUrl;
    sFileName = zTitle;
    sFileName[strlen(sFileName.c_str())-10] = '\0';
    pcParentWindow = pcParent;

    pcStringView = new StringView(Rect(10,10,80,20),"_add_as","Add as:");
    AddChild(pcStringView);

    pcTextView = new TextView(Rect(60, 5, 300, 27), "_add_txt","");
    pcTextView->Set(sFileName.c_str());
    AddChild(pcTextView);


    pcOk = new Button(Rect(105, 40, 155, 65), "_ok_but","Add",new Message(ID_OK_FAV_WINDOW));
    AddChild(pcOk);

    pcCancel = new Button(Rect(185, 40, 230, 65), "_cancel_but", "Cancel", new Message(ID_CANCEL_FAV_WINDOW));
    AddChild(pcCancel);
	 
}

void FavoriteWindow::CenterWindow( Window* pcWin )
{
    Rect cBounds = GetBounds();
    Rect cR = pcWin->GetFrame();

    MoveTo( cR.left + ( cR.Width() - cBounds.Width() ) / 2,
            cR.top  + ( cR.Height() - cBounds.Height() ) / 2 );
}

void FavoriteWindow::SaveFile()
{
    char zFileName[1024];
	sFileName=pcTextView->GetBuffer()[0];
    sprintf(zFileName, "%s/Settings/ABrowse/Favorites/%s.url",getenv("HOME"), sFileName.c_str());  
FILE* fout = fopen(zFileName, "w"); 
	fprintf(fout, "%s\n", sUrl.c_str());
    fclose(fout);
}

void FavoriteWindow::HandleMessage(Message* pcMessage)
{
    switch (pcMessage->GetCode())
    {
    case ID_CANCEL_FAV_WINDOW:
        Close();
        break;

    case ID_OK_FAV_WINDOW:
		{
        SaveFile();
        pcParentWindow->PostMessage(new Message(ID_UPDATE_FAVORITES), pcParentWindow);
        Close();
        break;
		}
    default:
        Window::PostMessage(pcMessage);
        break;
    }
}

int BrowserWindow::s_nWndCount = 0;

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

DropMenu* BrowserWindow::GetURLView()
{
    return (m_pcURLView);
}

KHTMLPart* BrowserWindow::GetPart()
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    return (hview->getParts());
}
void BrowserWindow::SwitchTab()
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    m_pcURLView->SetCurrentString(hview->getParts()->url().prettyURL().utf8().data());
    std::string cCaption=hview->getParts()->url().prettyURL().utf8().data();
	 std::string ctest = "- ABrowse";
    SetTitle(cCaption+ctest);
    std::string cURL = m_pcURLView->GetCurrentString();

    KHTMLView* pcView = hview->getParts()->view();

    pcView->SetResizeMask( CF_FOLLOW_ALL );
    Rect r = GetBounds();
	r.top += 2;
	r.bottom -= 89;
    pcView->SetFrame( r );

    pcView->MakeFocus();

    pcView->SetMsgTarget( Messenger( this) );

    pcView->show();
    hview->getParts()->browserExtension()->SetCallback(this);
    GoHistory(0);

}

void BrowserWindow::CreateNewTab()
{
    HTMLView* hview = new HTMLView;
    m_pcTabs->AppendTab("  (Untitled)  ",hview);

}
void BrowserWindow::CreateNewTab(const std::string& cURL, const KParts::URLArgs& cArgs)
{
    HTMLView* hview = new HTMLView(cURL,cArgs);
    m_pcTabs->AppendTab(" testing ",hview);
    std::string temp2 = cURL.substr(7,16);
    int i = m_pcTabs->GetTabCount()-1;
    std::string temp3 = temp2+"...";
    m_pcTabs->SetTabTitle(i,temp3.c_str() );
}
void BrowserWindow::DeleteTab()
{
    uint i = m_pcTabs->GetSelection();
    int nt = m_pcTabs->GetTabCount();
    if (nt > 1)
    {
        m_pcTabs->DeleteTab(i);
    }
    else
    {
     
    }

}
BrowserWindow::BrowserWindow( const Rect& cFrame, bool bLoadPos ) :
        Window( cFrame, "abrowse_wnd", "ABrowse", 0 )
{
    m_bSaveSize = bLoadPos;
    m_pcSearchDialog = NULL;
    m_bSearchCaseSensitive = false;
    pcFavoriteMessage = new Message();
    AddFavorites();
    atomic_add( &s_nWndCount, 1 );
    Rect	  cWndBounds = GetBounds();
    Rect t_wb = cWndBounds;
    t_wb.top=18;
    Rect menuwb = cWndBounds;
    menuwb.bottom=18;
    GlobalMutex::Lock();
    SetMutex( GlobalMutex::GetMutex() );
    GlobalMutex::PushLooper( this );
    m_pcTabs = new Tabs(t_wb);
    LayoutView* pcFrame = new LayoutView( t_wb, "frame_view" );

    VLayoutNode* pcRoot = new VLayoutNode( "root" );
    Menu *mmb = new Menu(menuwb,"mmb",ITEMS_IN_ROW);
    Menu *tempm = new Menu(Rect(0,0,0,0),"File",ITEMS_IN_COLUMN);
    tempm->AddItem("New Tab",new os::Message(ID_NEW_TAB));
    tempm->AddItem("Close Tab",new os::Message(ID_DELETE_TAB));

    mmb->AddItem(tempm);
    tempm=new Menu(Rect(0,0,0,0),"Edit",ITEMS_IN_COLUMN);
    tempm->AddItem("Copy",new os::Message(ID_COPY));
    tempm->AddItem("Paste",new os::Message(ID_PASTE));
    mmb->AddItem(tempm);
    pcFavoritesMenu = new FavoriteMenu("Favorites", pcFavoriteMessage,7000, true);
    mmb->AddItem(pcFavoritesMenu);
    mmb->SetTargetForItems(this);
    FrameView* pcNavBarFrame = new FrameView( Rect(0,0,0,0), "nav_bar_frame", "" );
    HLayoutNode* pcNavBar = new HLayoutNode( "nav_bar" );


     m_pcURLView = new DropMenu( Rect(0,0,0,0), "url_view");
	 m_pcURLView->SetSelectionMessage( new Message(ID_URLVIEW_CHANGE));
    m_pcURLView->SetEditMessage(  new Message(ID_URLVIEW_ENTER) );
	 m_pcURLView->SetMaxPreferredSize(cWndBounds.right);
    m_pcToolBar   = new ToolBar( Rect(0,0,0,0), "tool_bar" );
    m_pcStatusBar = new StatusBar( Rect(), "status_bar" );

    m_pcToolBar->AddButton( "back.png", os::Message( ID_PREV_URL ) );
    m_pcToolBar->AddButton( "forward.png", os::Message( ID_NEXT_URL ) );
    m_pcToolBar->AddButton( "gohome.png", os::Message( ID_GOHOME ) );
    m_pcToolBar->AddButton( "reload.png", os::Message( ID_RELOAD ) );
    m_pcToolBar->AddButton( "find.png", os::Message( ID_FIND ) );
    m_pcToolBar->AddButton( "editcopy.png", os::Message( ID_COPY ) );
    //m_pcToolBar->AddButton( "stop.png", os::Message( ID_STOP ) );
    pcNavBar->AddChild( m_pcToolBar )->SetBorders( Rect( 4, 2, 10, 2 ) );
    pcNavBar->AddChild( m_pcURLView );

    pcNavBarFrame->SetRoot( pcNavBar );

    pcRoot->AddChild( pcNavBarFrame );


    CreateNewTab();
    m_pcTabs->SetMessage(new Message(ID_TAB_CHANGED));
    pcRoot->AddChild( m_pcTabs );
    pcRoot->AddChild( m_pcStatusBar );
    pcFrame->SetRoot( pcRoot );

    AddChild( pcFrame );
    AddChild( mmb );
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    KHTMLView* pcView = hview->getParts()->view();
    pcView->SetResizeMask( CF_FOLLOW_ALL );
	Rect r = GetBounds();
	r.top+=2;
	r.bottom-=89;
    pcView->SetFrame( r );

    pcView->MakeFocus();

    pcView->SetMsgTarget( Messenger( this ) );

    pcView->show();
    hview->getParts()->browserExtension()->SetCallback(this);
    m_pcURLView->SetTarget( this );
    m_pcToolBar->SetTarget( Messenger( this ) );

    m_pcToolBar->EnableButton( BI_BACK, false );
    m_pcToolBar->EnableButton( BI_FORWARD, false );
    m_pcToolBar->EnableButton( BI_COPY, false );

    _Init();



    //AddTimer( this, 1, 100000, false );
    GlobalMutex::PopLooper();
    //GlobalMutex::Unlock();
}

void BrowserWindow::_Init()
{
    char*	pzHome = getenv( "HOME" );

    if ( m_bSaveSize && NULL != pzHome )
    {
        FILE*	hFile;
        char	zPath[ 256 ];

        strcpy( zPath, pzHome );
        strcat( zPath, "/config/abrowse.cfg" );

        hFile = fopen( zPath, "rb" );

        if ( NULL != hFile )
        {
            Rect cNewFrame;

            fread( &cNewFrame, sizeof( cNewFrame ), 1, hFile );
            fclose( hFile );
            SetFrame( cNewFrame );
        }
    }

    /*if ( hview->getHE() != NULL ) {
    	for ( uint i = 0 ; i < pcHistory->size() ; ++i ) {
     		HistoryEntry* pcEntry = new HistoryEntry;
     		*pcEntry = *(*pcHistory)[i];
     		pcEntry->m_cURL = (*pcHistory)[i]->m_cURL;
     		pcEntry->m_cState = (*pcHistory)[i]->m_cState;
     		hview->getHE().push_back( pcEntry );
    	}
    	hview->setCHP(m_cHistory.size() - 1);
    	m_pcToolBar->EnableButton( BI_BACK, true );
    }*/
}

/*void BrowserWindow::SetupMenus()
{
}
 
void BrowserWindow::SetupStatusBar()
{
}
 
void BrowserWindow::SetupToolBar()
{
}*/

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

BrowserWindow::~BrowserWindow()
{
    atomic_add( &s_nWndCount, -1 );
    Lock();
    GlobalMutex::PushLooper( this );
    int j;
    for(j=0;j<m_pcTabs->GetTabCount();j++)
    {
        m_pcTabs->DeleteTab(j);
    }
    delete m_pcTabs;
    GlobalMutex::PopLooper();
    Unlock();
}


void BrowserWindow::UpdateHistory()
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    HistoryEntry *pcCurrent = hview->HistoryCurrent();
    if ( pcCurrent == NULL )
    {
        pcCurrent = new HistoryEntry;
        hview->UpdateHistory(pcCurrent);
    }
    else
    {
        pcCurrent->m_cState.MakeEmpty();
    }
    GlobalMutex::PushLooper( this );
    if(hview->getParts()->url().prettyURL().utf8().data()!=NULL)
    {
        hview->getParts()->browserExtension()->saveState( &pcCurrent->m_cState );
        pcCurrent->m_cURL = hview->getParts()->url().prettyURL().utf8().data(); // m_pcURLView->GetBuffer()[0];
    }
    GlobalMutex::PopLooper();
}


void BrowserWindow::AddHistoryEntry()
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    hview->AddHistoryEntry();
    m_pcToolBar->EnableButton( BI_FORWARD, false );
    m_pcToolBar->EnableButton( BI_BACK, true );
}

void BrowserWindow::GoHistory( int nStep )
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    hview->GoHistory(nStep);
    HistoryEntry* pcCurrent = hview->HistoryCurrent();
    if ( pcCurrent != NULL )
    {
        GlobalMutex::PushLooper( this );
        hview->getParts()->browserExtension()->restoreState( pcCurrent->m_cState );
        m_pcURLView->SetCurrentString( pcCurrent->m_cURL.c_str());
        std::string temp2 = pcCurrent->m_cURL.substr(7,16);
        std::string temp3 = temp2+"...";
        m_pcTabs->SetTabTitle(i,temp3.c_str() );
        GlobalMutex::PopLooper();
    }
    if ( hview->getCHP() == 0 )
    {
        m_pcToolBar->EnableButton( BI_BACK, false );
    }
    else
    {
        m_pcToolBar->EnableButton( BI_BACK, true );
    }
    if ( hview->getCHP() >= int(hview->getHE().size()) - 1 )
    {
        m_pcToolBar->EnableButton( BI_FORWARD, false );
    }
    else
    {
        m_pcToolBar->EnableButton( BI_FORWARD, true );
    }
}

void dump_message( os::Message* pcMsg )
{
    int nCount = pcMsg->CountNames();

    for ( int i = 0 ; i < nCount ; ++i )
    {
        int nType;
        int nCount;

        pcMsg->GetNameInfo( pcMsg->GetName( i ).c_str(), &nType, &nCount );
        switch( nType )
        {
        case T_STRING:
            {
                std::string cStr;
                pcMsg->FindString( pcMsg->GetName( i ).c_str(), &cStr );
                printf( "%s: '%s'\n", pcMsg->GetName( i ).c_str(), cStr.c_str() );
                break;
            }
        default:
            printf( "%s: (Type=%d)\n", pcMsg->GetName( i ).c_str(), nType );
            break;

        }
    }
}
void BrowserWindow::OpenURL( const std::string& cURL, const KParts::URLArgs& cArgs )
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    GlobalMutex::PushLooper( this );
    m_pcURLView->SetCurrentString( cURL.c_str());
    std::string temp2 = cURL.substr(7,16);
    std::string temp3 = temp2+"...";
    m_pcTabs->SetTabTitle(i,temp3.c_str() );
    hview->openURL(cURL,cArgs);
    GlobalMutex::PopLooper();

}

void BrowserWindow::OpenURLRequest( const std::string &cURL, const KParts::URLArgs& cURLArgs )
{
    UpdateHistory();
    AddHistoryEntry();
    OpenURL( cURL, cURLArgs );
    UpdateHistory();
}

void BrowserWindow::OpenURLNotify()
{
    UpdateHistory();
}

void BrowserWindow::Completed()
{
    UpdateHistory();
}

void BrowserWindow::SetLocationBarURL( const std::string &cURL )
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    m_pcURLView->SetCurrentString( hview->getParts()->url().prettyURL().utf8().data());
    //    m_pcURLView->Set( cURL.c_str(), false );
}

void BrowserWindow::CreateNewWindow( const std::string& cURL, const KParts::URLArgs& cURLArgs )
{
    os::Rect cFrame( 0, 0, 799, 599 );

    cFrame += get_window_pos();

    //std::vector<HistoryEntry*> cHistory = m_cHistory;
    //cHistory.resize( m_nCurHistoryPos + 1 );
    BrowserWindow* pcNewWindow = new BrowserWindow( cFrame, false );
    if ( cURL.empty() == false )
    {
        pcNewWindow->OpenURL( cURL, cURLArgs );
        pcNewWindow->AddHistoryEntry();
    }
    pcNewWindow->Show( true );
    pcNewWindow->MakeFocus( true );
}

KParts::ReadOnlyPart* BrowserWindow::CreateNewWindow( const std::string& cURL, const KParts::URLArgs& cURLArgs, const KParts::WindowArgs& sWndArgs )
{
    os::Rect cFrame( 0, 0, 799, 599 );

    Point cPos;
    if ( sWndArgs.x == -1 || sWndArgs.y == -1 )
    {
        cPos = get_window_pos();
    }
    if ( sWndArgs.x != -1 )
    {
        cPos.x = sWndArgs.x;
    }
    if ( sWndArgs.y != -1 )
    {
        cPos.y  = sWndArgs.y;
    }
    if ( sWndArgs.width != -1 )
    {
        cFrame.right   = cFrame.left + sWndArgs.width - 1;
    }
    if ( sWndArgs.height != -1 )
    {
        cFrame.bottom  = cFrame.top + sWndArgs.height - 1;
    }
    cFrame += cPos;

    //    std::vector<HistoryEntry*> cHistory = m_cHistory;
    //  cHistory.resize( m_nCurHistoryPos + 1 );
    BrowserWindow* pcNewWindow = new BrowserWindow( cFrame, false );
    if ( cURL.empty() == false )
    {
        pcNewWindow->OpenURL( cURL, cURLArgs );
        pcNewWindow->AddHistoryEntry();
    }
    pcNewWindow->Show( true );
    pcNewWindow->MakeFocus( true );
    return( pcNewWindow->GetPart() );

}

void BrowserWindow::PopupMenu( KParts::BrowserExtension* /*pcSource*/, const std::string& cURL )
{
    m_pcPopupMenu     = new os::Menu( Rect( 0, 0, 10, 10 ), "popup", ITEMS_IN_COLUMN );
    //    Menu* pcSpeedMenu = new os::Menu( Rect( 0, 0, 10, 10 ), "Speed", ITEMS_IN_COLUMN );

    //    m_pcPopupMenu->AddItem( pcSpeedMenu );

    os::Message* pcMsg;
    m_pcPopupMenu->AddItem( new os::MenuItem( "New window", new os::Message( ID_NEW_WINDOW ) ) );
    m_pcPopupMenu->AddItem( new os::MenuItem( "Reload", new os::Message( ID_RELOAD ) ) );
	m_pcPopupMenu->AddItem( new os::MenuItem("Add to Favorites", new os::Message( ID_ADD_TO_FAVORITES)));
    if ( cURL.empty() == false )
    {
        m_pcPopupMenu->AddItem( new os::MenuSeparator() );
        pcMsg = new os::Message ( ID_OPEN_TAB );
        pcMsg->AddString( "url", cURL);
        m_pcPopupMenu->AddItem(new os::MenuItem( "Open link in new tab", pcMsg));
	
	  	pcMsg = new os::Message (ID_ADD_LINK_TO_FAVORITES);
	   pcMsg->AddString( "url", cURL);
    	m_pcPopupMenu->AddItem(new os::MenuItem( "Add link to Favorites", pcMsg));

	    pcMsg = new os::Message( ID_OPEN_LINK );
        pcMsg->AddString( "url", cURL );
        m_pcPopupMenu->AddItem( new os::MenuItem( "Open link in new window", pcMsg ) );

        pcMsg = new os::Message( ID_SAVE_LINK );
        pcMsg->AddString( "url", cURL );
        m_pcPopupMenu->AddItem( new os::MenuItem( "Save link as...", pcMsg ) );

        pcMsg = new os::Message( ID_COPY_LINK_LOCATION );
        pcMsg->AddString( "url", cURL );
        m_pcPopupMenu->AddItem( new os::MenuItem( "Copy link location", pcMsg ) );
    }
    //    m_pcPopupMenu->AddItem( new os::MenuItem( "View document source", new os::Message( ID_VIEW_DOC_SRC ) ) );
    //    pcMsg = new os::Message( ID_VIEW_FRAME_SRC );
    //    pcMsg->AddPointer( "browser", pcSource );
    //    m_pcPopupMenu->AddItem( new os::MenuItem( "View frame source",    new os::Message( ID_VIEW_FRAME_SRC ) ) );
    m_pcPopupMenu->AddItem( new os::MenuSeparator() );
    m_pcPopupMenu->AddItem( new os::MenuItem( "Quit", new os::Message( M_QUIT ) ) );

    m_pcPopupMenu->SetTargetForItems( this );

    os::Point cMousePos;
    m_pcStatusBar->GetMouse( &cMousePos, NULL );
    m_pcPopupMenu->Open( m_pcStatusBar->ConvertToScreen( cMousePos ) );
}

void BrowserWindow::LoadingProgress( int /*percent*/ )
{}
void BrowserWindow::SpeedProgress( int /*bytesPerSecond*/ )
{}
void BrowserWindow::InfoMessage( const std::string& cString )
{
    m_pcStatusBar->SetStatus( cString );
}

void BrowserWindow::SetWindowCaption( KParts::ReadOnlyPart* pcPart, const std::string& cCaption )
{
    uint i = m_pcTabs->GetSelection();
    HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
    if ( pcPart == hview->getParts() )
    {
        SetTitle( cCaption + " - ABrowse" );
    }
}

void BrowserWindow::SetStatusBarText( const std::string& cString )
{
    m_pcStatusBar->SetStatus( cString );
}


void BrowserWindow::SelectionInfo( const std::string& cText )
{
    m_cSelectedText = cText;
    m_pcToolBar->EnableButton( BI_COPY, m_cSelectedText.empty() == false );
}

bool BrowserWindow::BeginDownload( KParts::DownloadStream* psStream )
{
    DownloadNode* psNode = new DownloadNode;

    psNode->m_nStartTime     = get_system_time();
    psNode->m_cURL           = psStream->m_cURL;
    psNode->m_cPreferredName = psStream->m_cPreferredName;
    psNode->m_nContentSize   = psStream->m_nContentSize;

    try
    {
        psNode->m_pcTmpFile = new os::TempFile( "ab-" );
    }
    catch( ... )
    {
        printf( "Error: BrowserWindow::BeginDownload() failed to create temporary file\n" );
        return( false );
    }

    Message cOkMsg( ID_START_DOWNLOAD );
    Message cCancelMsg( ID_CANCEL_DOWNLOAD );

    cOkMsg.AddPointer( "node", psNode );
    cCancelMsg.AddPointer( "node", psNode );

    psNode->m_psDownloadStream = psStream;
    psNode->m_pcRequester = new DownloadRequester( Rect( 250, 200, 599, 300 ),
                            "File Download",
                            psStream->m_cURL,
                            psStream->m_cPreferredName,
                            psStream->m_cMimeType,
                            psStream->m_nContentSize,
                            Messenger( this ),
                            cOkMsg,
                            cCancelMsg );
    psNode->m_pcRequester->Show();
    psStream->m_pUserData = psNode;
    return( true );
}




t_List BrowserWindow::GetFavorites()
{

    t_List t_list;
    string zName;
    char zDir[1024];
    sprintf(zDir, "%s/Settings/ABrowse/Favorites/",getenv("HOME"));
    //cout << zDir << endl;
    Directory* pcDir = new Directory();

    if(pcDir->SetTo(zDir)==0)
    {
        while (pcDir->GetNextEntry(&zName))
            if (!(zName.find( ".url",0)==string::npos))
            {
                zName[strlen(zName.c_str())-4] = '\0';
                t_list.push_back(zName);
            }

    }
    delete pcDir;
    return (t_list);
}

void BrowserWindow::AddFavorites()
{
    t_List t_FavoriteList = GetFavorites();


    for (uint32 n = 0; n < t_FavoriteList.size(); n++)
    {
		//cout<<t_FavoriteList[n].c_str()<<endl;
		pcFavoriteMessage->AddString(t_FavoriteList[n].c_str(), t_FavoriteList[n]);
    }
}


void BrowserWindow::UpdateFavorites()
{
    pcFavoritesMenu->ClearHistory();
    pcFavoriteMessage->MakeEmpty();
    AddFavorites();
    pcFavoritesMenu->AddItems();
    pcFavoritesMenu->SetTarget(this);
}

bool BrowserWindow::HandleDownloadData( KParts::DownloadStream* psStream, const void* pData, int nLen )
{
    DownloadNode* psNode = static_cast<DownloadNode*>(psStream->m_pUserData);
    if ( psNode == NULL || psNode->m_bCanceled || (psNode->m_pcTmpFile == NULL && psNode->m_pcFile == NULL) )
    {
        return( false );
    }
    int nBytesWritten;

    if ( psNode->m_pcFile != NULL )
    {
        nBytesWritten = psNode->m_pcFile->Write( pData, nLen );
    }
    else
    {
        nBytesWritten = psNode->m_pcTmpFile->Write( pData, nLen );
    }
    if ( nBytesWritten != nLen )
    {
        (new os::Alert( "Error: Failed to create file!",
                        os::String().Format( "Failed to write %df bytes to\n%s\n\nError: %s\n", psNode->m_cPath.c_str(), strerror(errno) ), 0,
                        "Sorry!", NULL ))->Go(NULL);
        psNode->m_bCanceled = true;

        if ( psNode->m_pcProgressDlg != NULL )
        {
            psNode->m_pcProgressDlg->Terminate();
            psNode->m_pcProgressDlg = NULL;
        }
        if ( psNode->m_pcRequester == NULL )
        {
            delete psNode->m_pcTmpFile;
            delete psNode->m_pcFile;
            if ( psNode->m_cPath.empty() == false )
            {
                unlink( psNode->m_cPath.c_str() );
            }
            delete psNode;
        }
        else
        {
            psNode->m_bDone = true;
        }

    }
    else
    {
        psNode->m_nBytesReceived += nLen;
        if ( psNode->m_pcProgressDlg != NULL )
        {
            psNode->m_pcProgressDlg->UpdateProgress( psNode->m_nBytesReceived );
        }
    }
    return( true );
}

void BrowserWindow::DownloadFinished( KParts::DownloadStream* psStream, int nError )
{
    DownloadNode* psNode = static_cast<DownloadNode*>(psStream->m_pUserData);
    if ( psNode == NULL || psNode->m_bCanceled || (psNode->m_pcTmpFile == NULL && psNode->m_pcFile == NULL) )
    {
        return;
    }

    if ( psNode->m_pcProgressDlg != NULL )
    {
        psNode->m_pcProgressDlg->Terminate();
        psNode->m_pcProgressDlg = NULL;
    }
    if ( psNode->m_pcRequester == NULL )
    {
        delete psNode->m_pcTmpFile;
        delete psNode->m_pcFile;
        delete psNode;
    }
    else
    {
        psNode->m_bDone = true;
    }
}

void BrowserWindow::HandleMessage( Message* pcMsg )
{
    switch( pcMsg->GetCode() )
    {
	 
	 case ID_URLVIEW_CHANGE:
		{
		int32 check;
		pcMsg->FindInt32("selection",&check);
		std::string next = m_pcURLView->GetItem(check);
		Message* pMs = new Message(ID_URL_CHANGED);
		pMs->AddString("url",next);
		pMs->AddInt32("events",os::TextView::EI_ENTER_PRESSED);
		DispatchMessage(pMs,this);
		break;
		}
	 case ID_URLVIEW_ENTER:
	{
			bool check;
			pcMsg->FindBool("final",&check);
			if(check){
 			  Message* pMs = new Message(ID_URL_CHANGED);
            pMs->AddString("url",m_pcURLView->GetCurrentString());
			  pMs->AddInt32("events", os::TextView::EI_ENTER_PRESSED );//0x0002);
			  DispatchMessage(pMs,this);
				if(m_pcURLView->GetItemCount()<10){
			  m_pcURLView->InsertItem(0,m_pcURLView->GetCurrentString().c_str());
				}
				else{
					m_pcURLView->DeleteItem(10);
					m_pcURLView->InsertItem(0,m_pcURLView->GetCurrentString().c_str());
					}
				}
				break;
	}
    case ID_RECENT_ITEM:
        {
            char pzFile[1024];
            char pzLine[1024];
            sprintf(pzFile,"%s/Settings/ABrowse/Favorites/%s.url",getenv("HOME"),pcFavoritesMenu->GetClicked(pcMsg));
            FILE* pFile = fopen(pzFile, "r");
            fgets(pzLine, 1024, pFile);
            fclose(pFile);
            m_pcURLView->Clear();
            m_pcURLView->SetCurrentString(pzLine);
            Message* pMs = new Message(ID_URL_CHANGED);
            pMs->AddString("url", pzLine);
            pMs->AddInt32("events", os::TextView::EI_ENTER_PRESSED );//0x0002);
            DispatchMessage(pMs, this);
            break;
        }
	 case ID_MANAGE_FAVORITES:
	{
	//	ManageFavorites* win = new ManageFavorites(GetFavorites(),this);
	//	win->CenterWindow(this);
	//    win->Show();
	//    win->MakeFocus();
		break;	
	}

    case ID_ADD_TO_FAVORITES:
        {
            std::string zUrl;
			  zUrl = m_pcURLView->GetCurrentString();
			  pcFavoriteWindow = new FavoriteWindow(zUrl, GetTitle() ,this);
		     pcFavoriteWindow->CenterWindow(this);
            pcFavoriteWindow->Show();
            pcFavoriteWindow->MakeFocus();
            break;
        }
	case ID_ADD_LINK_TO_FAVORITES:
		{
		std::string cURL;
		pcMsg->FindString("url",&cURL);
		std::string cURL2 = cURL.substr(7,cURL.length());
		int index = cURL2.find('/');
		while(index !=std::string::npos){
			cURL2.replace(index,1,'-');
			index=cURL2.find('/');
		}
		std::string temp = " - ABrowse";
		std::string temp2 = cURL2+temp;
		pcFavoriteWindow = new FavoriteWindow(cURL,temp2,this);
		pcFavoriteWindow->CenterWindow(this);
		pcFavoriteWindow->Show();
		pcFavoriteWindow->MakeFocus();
		
		break;
	}
    case ID_UPDATE_FAVORITES:
	        UpdateFavorites();
        break;

    case ID_TAB_CHANGED:
        {
            SwitchTab();
            break;
        }
    case ID_NEW_TAB:
        {
            CreateNewTab();
            break;
        }
    case ID_DELETE_TAB:
        {
            DeleteTab();
            break;
        }
    case ID_URL_CHANGED:
        {
			 
         uint32 nEvents = 0;
            pcMsg->FindInt( "events", &nEvents );
            uint i = m_pcTabs->GetSelection();
            HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
            if ( nEvents & os::TextView::EI_ESC_PRESSED )
            {
                m_pcURLView->SetCurrentString(hview->getParts()->url().prettyURL().utf8().data() );
                hview->getParts()->view()->MakeFocus();
                break;
            }
            if ( nEvents & os::TextView::EI_ENTER_PRESSED )
            {
                std::string cURL = m_pcURLView->GetCurrentString();

                for ( uint i = 0 ; i <= cURL.size() ; ++i )
                {
                    if ( i == cURL.size() || isalnum( cURL[i] ) == false )
                    {
                        if ( i == cURL.size() || cURL[i] != ':' )
                        {
                            std::string cTmp = cURL;
                            cURL = "http://";
                            for ( uint j = 0 ; j < cTmp.size() ; ++j )
                            {
                                if ( cTmp[j] != '/' )
                                {
                                    cURL.insert( cURL.end(), cTmp.begin() + j, cTmp.end() );
                                    break;
                                }
                            }
                        }
                        break;
                    }
                }

                hview->getParts()->view()->MakeFocus();
                OpenURLRequest( cURL, KParts::URLArgs() );
            }
            break;
        }
    case ID_ACTIVATE_URLEDIT:
        m_pcURLView->MakeFocus( true );
        //m_pcURLView->SetCursor( -1, 0 );
        //m_pcURLView->SelectAll();
        break;
    case ID_RELOAD:
        {
            uint i = m_pcTabs->GetSelection();
            HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
            KParts::URLArgs cArgs;
            cArgs.reload = true;
            cArgs.xOffset = hview->getParts()->browserExtension()->xOffset();
            cArgs.yOffset = hview->getParts()->browserExtension()->yOffset();
            OpenURL( m_pcURLView->GetCurrentString(), cArgs );
            break;
        }
    case ID_NEW_WINDOW:
        {
            UpdateHistory();
            BrowserWindow* pcNewWindow = new BrowserWindow( GetFrame() + os::Point( 30.0f, 30.0f ), false );
            pcNewWindow->GoHistory(0);
            pcNewWindow->Show( true );
            pcNewWindow->MakeFocus( true );
            break;
        }
    case ID_OPEN_TAB:
        {
            std::string cURL;
            pcMsg->FindString("url",&cURL);
            CreateNewTab(cURL,KParts::URLArgs());
            break;
        }
    case ID_OPEN_LINK:
        {
            std::string cURL;
            pcMsg->FindString( "url", &cURL );
            // std::vector<HistoryEntry*> cHistory = m_cHistory;
            //cHistory.resize( m_nCurHistoryPos + 1 );
            BrowserWindow* pcNewWindow = new BrowserWindow( GetFrame() + os::Point( 30.0f, 30.0f ), false );
            pcNewWindow->OpenURL( cURL, KParts::URLArgs() );
            pcNewWindow->AddHistoryEntry();
            pcNewWindow->Show( true );
            pcNewWindow->MakeFocus( true );
            break;
        }
    case ID_SAVE_LINK:
        {
            uint i = m_pcTabs->GetSelection();
            HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
            std::string cURL;
            pcMsg->FindString( "url", &cURL );

            hview->getParts()->browserExtension()->SaveURL( cURL, "Save link as..." );
            break;
        }
    case ID_COPY_LINK_LOCATION:
        {
            std::string cURL;
            pcMsg->FindString( "url", &cURL );

            os::Clipboard cClipboard;
            cClipboard.Lock();
            cClipboard.Clear();
            os::Message* pcData = cClipboard.GetData();
            pcData->AddString( "text/plain", cURL );
            cClipboard.Commit();
            cClipboard.Unlock();
            break;
        }
    case ID_PASTE:
        {
            GetURLView()->Clear();
            //GetURLView()->Paste(NULL);
            break;
        }
    case ID_COPY:
        {
            if ( m_cSelectedText.empty() )
            {
                break;
            }
            os::Clipboard cClipboard;
            cClipboard.Lock();
            cClipboard.Clear();
            os::Message* pcData = cClipboard.GetData();
            pcData->AddString( "text/plain", m_cSelectedText );
            cClipboard.Commit();
            cClipboard.Unlock();
            break;
        }
    case ID_FIND:
        if ( m_pcSearchDialog == NULL )
        {
            uint i = m_pcTabs->GetSelection();
            HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
            hview->getParts()->findTextBegin();
            m_pcSearchDialog = new FindDialog( os::Messenger(this), os::Message(ID_DO_SEARCH), os::Message(ID_SEARCH_CLOSED) );
            m_pcSearchDialog->Show();
            m_pcSearchDialog->MakeFocus();
        }
        break;
    case ID_DO_SEARCH:
        {
            bool bFromTop = false;
            pcMsg->FindBool( "from_top", &bFromTop );
            pcMsg->FindBool( "case_sensitive", &m_bSearchCaseSensitive );
            pcMsg->FindString( "string", &m_cLastSearchStr );

            KHTMLView* pcActiveView = dynamic_cast<KHTMLView*>(GetFocusChild());
            uint i = m_pcTabs->GetSelection();
            HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
            KHTMLPart* pcPart = hview->getParts();
            if ( pcActiveView != NULL )
            {
                pcPart = pcActiveView->part();
            }
            if ( bFromTop )
            {
                pcPart->findTextBegin();
            }
            pcPart->findTextNext( m_cLastSearchStr.c_str(), true, m_bSearchCaseSensitive );
            break;
        }
    case ID_REPEAT_FIND:
        {
            uint i = m_pcTabs->GetSelection();
            HTMLView* hview = (HTMLView*)m_pcTabs->GetTabView(i);
            if ( m_cLastSearchStr.empty() == false )
            {
                KHTMLView* pcActiveView = dynamic_cast<KHTMLView*>(GetFocusChild());

                KHTMLPart* pcPart =hview->getParts();
                if ( pcActiveView != NULL )
                {
                    pcPart = pcActiveView->part();
                }
                pcPart->findTextNext( m_cLastSearchStr.c_str(), true, m_bSearchCaseSensitive );
            }
            else if ( m_pcSearchDialog == NULL )
            {
                hview->getParts()->findTextBegin();
                m_pcSearchDialog = new FindDialog( os::Messenger(this), os::Message(ID_DO_SEARCH), os::Message(ID_SEARCH_CLOSED) );
                m_pcSearchDialog->Show();
                m_pcSearchDialog->MakeFocus();
            }
            break;
        }
    case ID_SEARCH_CLOSED:
        m_pcSearchDialog = NULL;
        break;
    case ID_GOHOME:
        UpdateHistory();
        AddHistoryEntry();
        OpenURL( "http://syllable.sourceforge.net/", KParts::URLArgs() );
        UpdateHistory();
        break;
    case ID_PREV_URL:
        GoHistory( -1 );
        break;
    case ID_NEXT_URL:
        GoHistory( 1 );
        break;
    case ID_START_DOWNLOAD:
        {
            DownloadNode* psNode = NULL;
            std::string   cPath;
            pcMsg->FindPointer( "node", (void**) &psNode );
            pcMsg->FindString( "path", &cPath );


            if ( psNode == NULL )
            {
                break; // Should never happen
            }
            psNode->m_cPath = cPath;
            const std::string cDstDir = os::Path( cPath.c_str() ).GetDir();

            try
            {
                os::FSNode cDirNode( cDstDir );
                if ( cDirNode.GetDev() == psNode->m_pcTmpFile->GetDev() )
                {
                    rename( psNode->m_pcTmpFile->GetPath().c_str(), cPath.c_str() );
                    psNode->m_pcTmpFile->Detatch(); // Make sure the destructor don't do anything silly.
                }
                else
                {
                    psNode->m_pcFile = new os::File( cPath, O_WRONLY | O_CREAT );
                    psNode->m_pcTmpFile->Seek( 0, SEEK_SET );
                    char anBuffer[32*1024];
                    for (;;)
                    {
                        int nLen = psNode->m_pcTmpFile->Read( anBuffer, sizeof(anBuffer) );
                        if ( nLen < 0 )
                        {
                            throw os::errno_exception( "Failed to copy temporary file" );
                        }
                        if ( psNode->m_pcFile->Write( anBuffer, nLen ) != nLen )
                        {
                            throw os::errno_exception( "Failed to copy temporary file" );
                        }
                        if ( nLen < int(sizeof(anBuffer)) )
                        {
                            break;
                        }
                    }
                    delete psNode->m_pcTmpFile;
                    psNode->m_pcTmpFile = NULL;
                }
            }
            catch(...)
            {
                (new os::Alert( "Error: Failed to create file!",
                                os::String().Format( "Failed to create '%s'", cPath.c_str() ), 0,
                                "Sorry!", NULL ))->Go(NULL);
            }

            psNode->m_pcRequester = NULL;

            if ( psNode->m_bDone == false )
            {
                Message cCancelMsg( ID_CANCEL_DOWNLOAD );
                cCancelMsg.AddPointer( "node", psNode );

                psNode->m_pcProgressDlg = new DownloadProgress( os::Rect( 100, 100, 449, 349 ), os::String().Format( "Download: %s", cPath.c_str() ),
                                          psNode->m_cURL,
                                          cPath,
                                          psNode->m_nContentSize,
                                          psNode->m_nStartTime,
                                          os::Messenger( this ),
                                          cCancelMsg );

                psNode->m_pcProgressDlg->Show();
            }
            else
            {
                delete psNode->m_pcTmpFile;
                delete psNode->m_pcFile;
                delete psNode;
            }
            break;
        }
    case ID_CANCEL_DOWNLOAD:
        {
            DownloadNode* psNode = NULL;
            pcMsg->FindPointer( "node", (void**) &psNode );

            if ( psNode == NULL )
            {
                break; // Should never happen
            }
            if ( psNode->m_pcProgressDlg != NULL )
            {
                psNode->m_pcProgressDlg->Terminate();
                psNode->m_pcProgressDlg = NULL;
            }
            if ( psNode->m_cPath.empty() == false )
            {
                unlink( psNode->m_cPath.c_str() );
            }
            if ( psNode->m_bDone == false )
            {
                psNode->m_pcRequester = NULL;
                psNode->m_bCanceled = true;
                delete psNode->m_pcTmpFile;
                psNode->m_pcTmpFile = NULL;
                delete psNode->m_pcFile;
                psNode->m_pcFile = NULL;
            }
            else
            {
                delete psNode->m_pcTmpFile;
                delete psNode->m_pcFile;
                delete psNode;
            }
            break;
        }
    default:
        Window::HandleMessage( pcMsg );
        break;
    }
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool BrowserWindow::OkToQuit( void )
{
    if ( m_pcSearchDialog != NULL )
    {
        if ( m_pcSearchDialog->SafeLock() >= 0 )
        {
            m_pcSearchDialog->PostMessage( M_QUIT );
            m_pcSearchDialog->Unlock();
            m_pcSearchDialog = NULL;
        }
    }
    if ( s_nWndCount == 1 )
    {
        if ( m_bSaveSize )
        {
            char*	pzHome = getenv( "HOME" );

            if ( NULL != pzHome )
            {
                FILE*	hFile;
                char	zPath[ 256 ];

                strcpy( zPath, pzHome );
                strcat( zPath, "/config/abrowse.cfg" );

                hFile = fopen( zPath, "wb" );

                if ( hFile != NULL )
                {
                    Rect cFrame = GetFrame();
                    fwrite( &cFrame, sizeof( cFrame ), 1, hFile );
                    fclose( hFile );
                }
            }
        }
        Application::GetInstance()->PostMessage( M_QUIT );
    }
    return( true );
}






void print_msg( QtMsgType, const char * /*pzMsg*/ )
{
    //    printf( "QWarning: %s\n", pzMsg );
}


int main()
{
    Application* pcMyApp;

    qInstallMsgHandler( print_msg );

    pcMyApp = new MyApp( "application/x-vnd.KHS-abrowse" );

    pcMyApp->Run();
    return( 0 );
}









